from datetime import datetime
from typing import Any, List, Optional

from clockify.model.base_model import BaseModel
from clockify.model.user_model import User
from clockify.model.tag_model import Tag
from clockify.model.task_model import Task
from clockify.model.project_model import Project


class TimeInterval(BaseModel):
    start: Optional[str]
    end: Optional[str]
    duration: Optional[str]


class HourlyRate(BaseModel):
    amount: Optional[float]
    currency: Optional[str]


class TimeEntry(BaseModel):
    id_: Optional[str]
    description: str
    tags: Optional[List[Tag]]
    tag_ids: Optional[List[str]]
    user: Optional[User]
    user_id: Optional[str]
    billable: Optional[bool]
    task: Optional[Task]
    task_id: Optional[str]
    project: Optional[Project]
    project_id: Optional[str]
    time_interval: Optional[TimeInterval]
    workspace_id: Optional[str]
    hourly_rate: Optional[HourlyRate]
    custom_field_values: List[Any]
    is_locked: Optional[bool]

    class Config:
        fields = {
            "id_": "id",
            "tag_ids": "tagIds",
            "user_id": "userId",
            "task_id": "taskId",
            "project_id": "projectId",
            "time_interval": "timeInterval",
            "workspace_id": "workspaceId",
            "hourly_rate": "hourlyRate",
            "custom_field_values": "customFieldValues",
            "is_locked": "isLocked",
        }


class TimeEntryGetParams(BaseModel):
    description: Optional[str]
    start: Optional[str]
    end: Optional[str]
    project: Optional[str]
    task: Optional[str]
    tags: Optional[List[str]]
    project_required: Optional[bool]
    task_required: Optional[bool]
    hydrated: Optional[bool]
    in_progress: Optional[bool]
    page: Optional[int] = 1
    page_size: Optional[int] = 50


class CreateTimeEntryDTO(BaseModel):
    start: datetime
    end: datetime
    billable: bool = True
    description: str
    project_id: Optional[str]
    task_id: Optional[str]
    tag_ids: Optional[List[str]]

    class Config:
        fields = {"project_id": "projectId", "task_id": "taskId", "tag_ids": "tagIds"}



from typing import List
from clockify.model.time_entry_model import (
    TimeEntry,
    TimeEntryGetParams,
    CreateTimeEntryDTO,
)
from clockify.config import BASE_URL
from clockify.wrapper import Wrapper


class TimeEntryApi(Wrapper):
    def get_time_entries(
        self,
        workspace_id: str,
        user_id: str,
        params: TimeEntryGetParams = TimeEntryGetParams(),
    ) -> List[TimeEntry]:
        """Return list of TimeEntry object.

        Args:
            workspace_id (str): ID of the clockify workspace.
            user_id (str): ID of the clockify user.
            params (TimeEntryGetParams, optional): Path params. Defaults to TimeEntryGetParams().

        Returns:
            List[TimeEntry]: List of TimeEntry objects.
        """
        url = f"{BASE_URL}/workspaces/{workspace_id}/user/{user_id}/time-entries"
        return self._get_list(url, TimeEntry, params)

    def get_time_entry(self, workspace_id: str, time_entry_id: str) -> TimeEntry:
        """Return one TimeEntry object

        Args:
            workspace_id (str): ID of the clockify workspace.
            time_entry_id (str): ID of the clockify time-entry.

        Returns:
            TimeEntry: TimeEntry object.
        """
        url = f"{BASE_URL}/workspaces/{workspace_id}/time-entries/{time_entry_id}"
        return self._get_one(url, TimeEntry)

    def add_time_entry(
        self, workspace_id: str, user_id: str, time_entry: CreateTimeEntryDTO
    ) -> TimeEntry:
        """Add new TimeEntry and return it as a TimeEntry object.

        Args:
            workspace_id (str): ID of the clockify workspace.
            user_id (str): ID of the clockify user.
            time_entry (CreateTimeEntryDTO): TimeEntry to create.

        Returns:
            TimeEntry: Added TimeEntry object.
        """
        url = f"{BASE_URL}/workspaces/{workspace_id}/user/{user_id}/time-entries"
        return self._create_one(url, time_entry, TimeEntry)



from typing import List
from clockify.model.project_model import Project, ProjectGetParams
from clockify.config import BASE_URL
from clockify.wrapper import Wrapper


class ProjectApi(Wrapper):
    def get_projects(
        self, workspace_id: str, params: ProjectGetParams = ProjectGetParams()
    ) -> List[Project]:
        """Return list of Project objects.

        Args:
            workspace_id (str): ID of the clockify workspace.
            params (ProjectGetParams, optional): Path parameters. Defaults to ProjectGetParams().

        Returns:
            List[Project]: List of Project Objects
        """
        url = self.__url(workspace_id)
        return self._get_list(url, Project, params)

    def get_project(self, workspace_id: str, project_id: str) -> Project:
        """Return one Project object.

        Args:
            workspace_id (str): ID of the clockify workspace.
            project_id (str): ID of the clockify project.

        Returns:
            Project: Project object.
        """
        url = self.__url(workspace_id, project_id)
        return self._get_one(url, Project)

    def create_project(self, project: Project) -> Project:
        """Create one Project and return it as a Project object.

        Args:
            project (Project): Clockify Project to create.

        Returns:
            Project: Created Project object.
        """
        url = self.__url(project.workspace_id, project.id_)
        return self._create_one(url, project, Project)

    def delete_project(self, workspace_id: str, project_id: str) -> Project:
        """Delete a Project and return it as a Project object.

        Args:
            workspace_id (str): ID of the clockify workspace.
            project_id (str): ID of the clockify project.

        Returns:
            Project: Deleted Project object.
        """
        url = self.__url(workspace_id, project_id)
        return self._delete_one(url, Project)

    def update_project(self, project: Project) -> Project:
        """Update a Project and return it as a Project object.

        Args:
            project (Project): Project to update.

        Returns:
            Project: Updated Project object.
        """
        url = self.__url(project.workspace_id, project.id_)
        return self._update_one(url, project, Project)

    def __url(self, workspace_id: str, project_id: str = None) -> str:
        url = f"{BASE_URL}/workspaces/{workspace_id}/projects"
        if project_id:
            url += f"/{project_id}"
        return url



from typing import List
from clockify.config import BASE_URL
from clockify.model.tag_model import Tag, TagGetParams
from clockify.wrapper import Wrapper


class TagApi(Wrapper):
    def get_tags(
        self, workspace_id: str, params: TagGetParams = TagGetParams()
    ) -> List[Tag]:
        """Return a list of Tag objects.

        Args:
            workspace_id (str): ID of the clockify workspace.
            params (TagGetParams, optional): Path parameters. Defaults to TagGetParams().

        Returns:
            List[Tag]: List of Tag objects.
        """
        url = self.__url(workspace_id)
        return self._get_list(url, Tag, params)

    def get_tag(self, workspace_id: str, tag_id: str) -> Tag:
        """Return one Tag object

        Args:
            workspace_id (str): ID of the clockify workspace.
            tag_id (str): ID of the clockify tag.

        Returns:
            Tag: Tag object.
        """
        url = self.__url(workspace_id, tag_id)
        return self._get_one(url, Tag)

    def create_tag(self, tag: Tag) -> Tag:
        """Create a Tag and return it as a Tag object.

        Args:
            tag (Tag): Tag to create.

        Returns:
            Tag: Created Tag object.
        """
        url = self.__url(tag.workspace_id)
        return self._create_one(url, tag, Tag)

    def delete_tag(self, workspace_id: str, tag_id: str) -> Tag:
        """Delete a Tag and return it as a Tag object

        Args:
            workspace_id (str): ID of the clockify workspace.
            tag_id (str): ID of the clockify tag.

        Returns:
            Tag: Deleted Tab object.
        """
        url = self.__url(workspace_id, tag_id)
        return self._delete_one(url, Tag)

    def update_tag(self, tag: Tag) -> Tag:
        """Update a Tag and return it as a Tag object

        Args:
            tag (Tag): Tag to update.

        Returns:
            Tag: Updated Tag object.
        """
        url = self.__url(tag.workspace_id, tag.id_)
        return self._update_one(url, tag, Tag)

    def __url(self, workspace_id: str, tag_id: str = None) -> str:
        url = f"{BASE_URL}/workspaces/{workspace_id}/tags"
        if tag_id:
            url += f"/{tag_id}"
        return url



from typing import Optional

from clockify.model.base_model import BaseModel


class Tag(BaseModel):
    id_: str = None
    name: str
    workspace_id: str
    archived: bool = False

    class Config:
        fields = {"id_": "id", "workspace_id": "workspaceId"}


class TagGetParams(BaseModel):
    name: Optional[str]
    archived: Optional[bool]
    page: int = 1
    page_size: int = 50

    class Config:
        fields = {"page_size": "page-size"}



from typing import Literal, Optional, List

from clockify.model.base_model import BaseModel


class HourlyRate(BaseModel):
    amount: Optional[str]
    currency: Optional[str]

    class Config:
        fields = {"ammount": "amount", "amount": "amount"}


class MemberShip(BaseModel):
    user_Id: Optional[str]
    hourly_rate: Optional[str]
    cost_rate: Optional[str]
    target_Id: Optional[str]
    membership_Type: Optional[str]
    membership_Status: Optional[str]

    class Config:
        fields = {
            "user_Id": "userId",
            "hourly_rate": "hourlyRate",
            "cost_rate": "costRate",
            "target_Id": "targetId",
            "membership_Type": "membershipType",
            "membership_Status": "membershipStatus",
        }


class Estimate(BaseModel):
    estimate: Optional[str]
    type: Optional[str]

    class Config:
        fields = {"estimate": "estimate", "type": "type"}


class TimeEstimate(BaseModel):
    estimate: Optional[str]
    type: Optional[str]
    reset_option: Optional[str]
    active: Optional[bool]
    include_non_billable: Optional[bool]

    class Config:
        fields = {
            "estimate": "estimate",
            "type": "type",
            "reset_option": "resetOption",
            "active": "active",
            "include_non_billable": "includeNonBillable",
        }


class Project(BaseModel):
    name: str
    workspace_id: str
    id_: Optional[str]
    hourly_rate: Optional[HourlyRate]
    client_id: Optional[str]
    billable: Optional[bool]
    memberships: Optional[List[MemberShip]]
    color: Optional[str]
    estimate: Optional[Estimate]
    archived: Optional[bool]
    duration: Optional[str]
    client_name: Optional[str]
    note: Optional[str]
    cost_rate: Optional[str]
    time_estimate: Optional[TimeEstimate]
    budget_estimate: Optional[str]
    template: Optional[bool]
    public_: Optional[bool]

    class Config:
        fields = {
            "id_": "id",
            "name": "name",
            "hourly_rate": "hourlyRate",
            "client_id": "clientId",
            "workspace_id": "workspaceId",
            "billable": "billable",
            "memberships": "memberships",
            "color": "color",
            "estimate": "estimate",
            "archived": "archived",
            "duration": "duration",
            "client_name": "clientName",
            "note": "note",
            "cost_rate": "costRate",
            "time_estimate": "timeEstimate",
            "budget_estimate": "budgetEstimate",
            "template": "template",
            "public_": "public",
        }


class ProjectGetParams(BaseModel):
    hydrated: Optional[bool]
    archived: Optional[bool]
    name: Optional[str]
    page: int = 1
    page_size: int = 50
    billable: Optional[bool]
    clients: Optional[List[str]]
    contains_client: Optional[bool]
    client_status: Optional[Literal["ACTIVE", "ARCHIVED"]]
    users: Optional[List[str]]
    contains_users: Optional[bool]
    user_status: Optional[Literal["ACTIVE", "ARCHIVED"]]
    is_template: Optional[bool]
    sort_column: Optional[Literal["NAME", "CLIENT_NAME", "DURATION"]]
    sort_order: Optional[Literal["ASCENDING", "DESCENDING"]]

    class Config:
        fields = {
            "page_size": "page-size",
            "contains_client": "contains-client",
            "client_status": "client-status",
            "contains_users": "contains-users",
            "user_status": "user-status",
            "is_template": "is-template",
            "sort_column": "sort-column",
            "sort_order": "sort-order",
        }



from typing import Literal, Optional

from clockify.model.base_model import BaseModel


class Client(BaseModel):
    id_: str = None
    name: str
    workspace_id: str
    archived: bool = False

    class Config:
        fields = {"id_": "id", "workspace_id": "workspaceId"}


class ClientGetParams(BaseModel):
    archived: bool = None
    name: str = None
    page: int = 1
    page_size: int = 50
    sort_column: Literal["NAME"] = None
    sort_order: Literal["ASCENDING", "DESCENDING"] = None

    class Config:
        fields = {
            "page_size": "page-size",
            "sort_column": "sort-column",
            "sort_order": "sort-order",
        }


class ClientUpdateParams(BaseModel):
    class Config:
        fields = {"archive_projects": "archive-projects"}



from clockify.model.user_model import User
from clockify.config import BASE_URL
from clockify.wrapper import Wrapper


class UserWrapper(Wrapper):
    def get_current_user(self) -> User:
        """Return current user as a User Object

        Returns:
            User: Current user object.
        """
        url = f"{BASE_URL}/user"
        return self._get_one(url, User)



from typing import List, Literal, Optional

from clockify.model.base_model import BaseModel


class CostRate_HourlyRate(BaseModel):
    amount: int
    currency: str


class Task(BaseModel):
    id_: str = None
    name: str
    project_id: str
    assignee_ids: List[str] = []
    estimate: str = None
    billable: bool = None
    hourly_rate: CostRate_HourlyRate = None
    cost_rate: CostRate_HourlyRate = None
    status: Literal["ACTIVE", "Done"] = None

    class Config:
        fields = {
            "id_": "id",
            "project_id": "projectId",
            "assignee_ids": "assigneIds",
            "hourly_rate": "hourlyRate",
            "cost_rate": "costRate",
        }


class TaskGetParams(BaseModel):
    is_active: Optional[bool]
    name: Optional[str]
    page: int = 1
    page_size: int = 50
    strict_name_search: Optional[bool]
    sort_column: Optional[Literal["ID", "NAME"]]
    sort_order: Optional[Literal["ASCENDING", "DESCENDING"]]

    class Config:
        fields = {
            "is_active": "is-active",
            "page_size": "page-size",
            "strict_name_search": "strict-name-search",
            "sort_column": "sort-column",
            "sort_order": "sort-order",
        }



import requests
from clockify.apis.project_api import ProjectApi
from clockify.apis.task_api import TaskApi
from clockify.apis.time_entry_api import TimeEntryApi
from clockify.apis.user import UserWrapper
from clockify.apis.client_api import ClientApi
from clockify.apis.tag_api import TagApi


class ClockifySession(
    UserWrapper,
):
    def __init__(self, key: str) -> None:
        """Initialize a Clockfiy Session

        Args:
            key (str): Your Clockify API Key
        """
        self.session = requests.Session()
        self.session.headers.update({"x-api-key": key})

        self.client = ClientApi(key)
        self.project = ProjectApi(key)
        self.tag = TagApi(key)
        self.task = TaskApi(key)
        self.time_entry = TimeEntryApi(key)



from typing import List
import requests

from clockify.model.base_model import BaseModel


class Wrapper:
    def __init__(self, key: str) -> None:
        self.session = requests.Session()
        self.session.headers.update({"x-api-key": key})

    def __get(self, url: str, query: dict = {}) -> dict:
        res = self.session.get(url, params=query)
        if res.status_code == 200:
            return res.json()
        else:
            raise requests.HTTPError(
                f"HTTP ERROR {res.status_code}: {res.reason} - {res.text}"
            )

    def __post(self, url: str, payload: dict) -> dict:
        res = self.session.post(url, json=payload)
        if res.status_code == 201:
            return res.json()
        else:
            raise requests.HTTPError(
                f"HTTP ERROR {res.status_code}: {res.reason} - {res.text}"
            )

    def __delete(self, url: str) -> dict:
        res = self.session.delete(url)
        if res.status_code == 200:
            return res.json()
        else:
            raise requests.HTTPError(
                f"HTTP ERROR {res.status_code}: {res.reason} - {res.text}"
            )

    def __put(self, url: str, payload: dict, params: dict) -> dict:
        res = self.session.put(url, json=payload, params=params)
        if res.status_code == 200:
            return res.json()
        else:
            raise requests.HTTPError(
                f"HTTP ERROR {res.status_code}: {res.reason} - {res.text}"
            )

    def _get_one(self, url: str, schema: BaseModel) -> BaseModel:
        res = self.__get(url)
        return schema(**res)

    def _get_list(
        self, url: str, schema: BaseModel, params: BaseModel = None
    ) -> List[BaseModel]:
        if params:
            params = params.json_dict()
        else:
            params = {}
        res = self.__get(url, params)
        return [schema(**r) for r in res]

    def _create_one(self, url: str, object: BaseModel, schema: BaseModel) -> BaseModel:
        res = self.__post(url, object.json_dict())
        return schema(**res)

    def _delete_one(self, url: str, schema: BaseModel) -> BaseModel:
        res = self.__delete(url)
        return schema(**res)

    def _update_one(
        self, url: str, object: BaseModel, schema: BaseModel, params: BaseModel = None
    ) -> BaseModel:
        if params:
            params = params.json_dict()
        else:
            params = {}
        res = self.__put(url, object.json_dict(), params)
        return schema(**res)



from bidict import bidict
from dataclasses import asdict
from enum import Enum
from clockify.dto import DTO


class Mapper:
    MAPPING_DICT = bidict()

    def dict_factory(self, data):
        def convert_value(obj):
            if isinstance(obj, Enum):
                return obj.value
            return obj

        return dict((k, convert_value(v)) for k, v in data)

    def to_api(self, input: DTO) -> dict:
        input_dict = asdict(input, dict_factory=self.dict_factory)
        return {self.MAPPING_DICT[k]: v for k, v in input_dict.items()}

    def to_dto(self, input_dict: dict) -> dict:
        return {self.MAPPING_DICT.inverse[k]: v for k, v in input_dict.items()}



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['clockify', 'clockify.apis', 'clockify.model']

package_data = \
{'': ['*']}

install_requires = \
['bidict>=0.22.0,<0.23.0',
 'marshmallow>=3.15.0,<4.0.0',
 'pydantic>=1.9.0,<2.0.0',
 'requests>=2.27.1,<3.0.0']

setup_kwargs = {
    'name': 'clockify-api',
    'version': '0.1.2',
    'description': "Python wrapper for Clockify's API.",
    'long_description': '![GitHub Workflow Status](https://img.shields.io/github/workflow/status/jpweijers/clockify-api/CI)\n[![Coverage Status](https://coveralls.io/repos/github/jpweijers/clockify-api/badge.svg?branch=main)](https://coveralls.io/github/jpweijers/clockify-api?branch=main)\n[![Documentation Status](https://readthedocs.org/projects/clockify-api/badge/?version=latest)](https://clockify-api.readthedocs.io/en/latest/?badge=latest)\n\n# Clockify\n\n## Documentation\n\n- [Package Documentation](clockify-api.readthedocs.io)\n- [Official Clocify API reference](https://clockify.me/developers-api)\n\n## Installation\n\n```bash\n# Pip\npip install clockify-api\n\n# Poetry\npoetry add clockify-api\n```\n\n## Example Usage\n\n```python\nfrom clockify.session import ClockifySession\n\nKEY = "YOUR_API KEY"\nWORKSPACE = "YOUR WORKSPACE ID"\n\nclockify_session = ClockifySession(KEY)\n\nprojects = clockify_session.project.get_projects(WORKSPACE)\n\nfor project in projects:\n    print(f"Project {project.name}, Client: {project.client_name}")\n```\n',
    'author': 'Jean-Paul Weijers',
    'author_email': 'jpweijers@users.noreply.github.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/jpweijers/clockify-api',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.5,<4.0',
}


setup(**setup_kwargs)



from typing import List
from clockify.model.task_model import Task, TaskGetParams
from clockify.config import BASE_URL
from clockify.wrapper import Wrapper


class TaskApi(Wrapper):
    def get_tasks(
        self,
        workspace_id: str,
        project_id: str,
        params: TaskGetParams = TaskGetParams(),
    ) -> List[Task]:
        """Get list of Task object.

        Args:
            workspace_id (str): ID of the clockify workspace.
            project_id (str): ID of the clockify project.
            params (TaskGetParams, optional): Path parameters. Defaults to TaskGetParams().

        Returns:
            List[Task]: List of Task objects.
        """
        url = self.__url(workspace_id, project_id)
        return self._get_list(url, Task, params)

    def get_task(self, workspace_id: str, project_id: str, task_id: str) -> Task:
        """Return one Task object

        Args:
            workspace_id (str): ID of the clockify workspace.
            project_id (str): ID of the clockify project.
            task_id (str): ID of the clockify task.

        Returns:
            Task: Task object.
        """
        url = self.__url(workspace_id, project_id, task_id)
        return self._get_one(url, Task)

    def create_task(self, workspace_id: str, task: Task) -> Task:
        """Create a task and return it as a Task object.

        Args:
            workspace_id (str): ID of the clockify workspace.
            task (Task): Task to create.

        Returns:
            Task: Created Task object.
        """
        url = self.__url(workspace_id, task.project_id, task.id_)
        return self._create_one(url, task, Task)

    def delete_task(self, workspace_id: str, project_id: str, task_id: str) -> Task:
        """Delete a task and return it as a Task object.

        Args:
            workspace_id (str): ID of the clockify workspace.
            project_id (str): ID of the clockify project.
            task_id (str): ID of the clockify task.

        Returns:
            Task: Deleted Task object.
        """
        url = self.__url(workspace_id, project_id, task_id)
        return self._delete_one(url, Task)

    def update_task(self, workspace_id: str, task: Task) -> Task:
        """Update a task and return it as a Task object.

        Args:
            workspace_id (str): ID of the clockify workspace.
            task (Task): Task to update.

        Returns:
            Task: Updated Task object.
        """
        url = self.__url(workspace_id, task.project_id, task.id_)
        return self._update_one(url, task, Task)

    def __url(self, workspace_id: str, project_id: str, task_id: str = None) -> str:
        url = f"{BASE_URL}/workspaces/{workspace_id}/projects/{project_id}/tasks"
        if task_id:
            url += f"/{task_id}"
        return url



from typing import List
from clockify.config import BASE_URL
from clockify.model.client_model import Client, ClientGetParams, ClientUpdateParams
from clockify.wrapper import Wrapper


class ClientApi(Wrapper):
    def get_clients(
        self, workspace_id: str, params: ClientGetParams = ClientGetParams()
    ) -> List[Client]:
        """Return a list of clients.

        Args:
            workspace_id (str): ID of the Clockify workspace.
            params (ClientGetParams, optional): Path Parameters. Defaults to ClientGetParams().

        Returns:
            List[Client]: List of Client objects
        """
        url = self.__url(workspace_id)
        return self._get_list(url, Client, params)

    def get_client(self, workspace_id: str, client_id: str) -> Client:
        """Return one client Object

        Args:
            workspace_id (str): ID of the clockify workspace
            client_id (str): Id of the clockify client

        Returns:
            Client: A Client object.
        """
        url = self.__url(workspace_id, client_id)
        return self._get_one(url, Client)

    def create_client(self, client: Client) -> Client:
        """Create one client and return it as a object.

        Args:
            client (Client): Client object.

        Returns:
            Client: Client object.
        """
        url = self.__url(client.workspace_id)
        return self._create_one(url, client, Client)

    def delete_client(self, workspace_id: str, client_id: str) -> Client:
        """Delete a Client, and return it as a Client object.

        Args:
            workspace_id (str): ID of the clockify workspace.
            client_id (str): ID of the clockify client.

        Returns:
            Client: Deleted Client object.
        """
        url = self.__url(workspace_id, client_id)
        return self._delete_one(url, Client)

    def update_client(
        self, client: Client, params: ClientUpdateParams = ClientUpdateParams()
    ) -> Client:
        """Update a Client and return it as a Client object.

        Args:
            client (Client): Client object.
            params (ClientUpdateParams, optional): Path parameters. Defaults to ClientUpdateParams().

        Returns:
            Client: Updated Client object.
        """
        url = self.__url(client.workspace_id, client.id_)
        return self._update_one(url, client, Client, params)

    def __url(self, workspace_id: str, client_id: str = None) -> str:
        """Return generated url

        Args:
            workspace_id (str): ID of the Clockify workspace.
            client_id (str, optional): ID of the Clockify Client. Defaults to None.

        Returns:
            str: Generated url
        """
        url = f"{BASE_URL}/workspaces/{workspace_id}/clients"
        if client_id:
            url += f"/{client_id}"
        return url



from typing import List, Literal, Optional

from clockify.model.base_model import BaseModel


class Membership(BaseModel):
    user_id: Optional[str]
    hourly_rate: Optional[float]
    cost_rate: Optional[float]
    target_id: Optional[str]
    membership_type: Optional[str]
    membership_status: Optional[str]

    class Config:
        fields = {
            "user_id": "userId",
            "hourly_rate": "hourlyRate",
            "cost_rate": "costRate",
            "target_id": "targetId",
            "membership_type": "membershipType",
            "membership_status": "membershipStatus",
        }


class SummaryReportSettings(BaseModel):
    group: Optional[str]
    subgroup: Optional[str]


class Settings(BaseModel):
    week_start: Optional[
        Literal[
            "MONDAY",
            "TUESDAY",
            "WEDNESDAY",
            "THURSDAY",
            "FRIDAY",
            "SATURDAY",
            "SUNDAY",
        ]
    ]
    time_zone: Optional[str]
    time_format: Optional[str]
    date_format: Optional[str]
    send_newsletter: Optional[str]
    weekly_updates: Optional[str]
    long_running: Optional[str]
    scheduled_reports: Optional[str]
    approval: Optional[str]
    pto: Optional[str]
    alerts: Optional[str]
    reminders: Optional[str]
    time_tracking_manual: Optional[str]
    summary_report_settings: Optional[SummaryReportSettings]
    is_compact_view_on: Optional[str]
    dashboard_selection: Optional[str]
    dashboard_view_type: Optional[str]
    dashboard_pin_to_top: Optional[str]
    project_list_collapse: Optional[str]
    collapse_all_project_lists: Optional[str]
    group_similar_entries_disabled: Optional[str]
    my_start_of_day: Optional[str]
    project_picker_task_filter: Optional[str]
    lang: Optional[str]
    multi_factor_enabled: Optional[str]
    theme: Optional[str]
    scheduling: Optional[str]

    class Config:
        fields = {
            "week_start": "weekStart",
            "time_zone": "timeZone",
            "time_format": "timeFormat",
            "date_format": "dateFormat",
            "send_newsletter": "sendNewsletter",
            "weekly_updates": "weeklyUpdates",
            "long_running": "longRunning",
            "scheduled_reports": "scheduledReports",
            "approval": "approval",
            "pto": "pto",
            "alerts": "alerts",
            "reminders": "reminders",
            "time_tracking_manual": "timeTrackingManual",
            "summary_report_settings": "summaryReportSettings",
            "is_compact_view_on": "isCompactViewOn",
            "dashboard_selection": "dashboardSelection",
            "dashboard_view_type": "dashboardViewType",
            "dashboard_pin_to_top": "dashboardPinToTop",
            "project_list_collapse": "projectListCollapse",
            "collapse_all_project_lists": "collapseAllProjectLists",
            "group_similar_entries_disabled": "groupSimilarEntriesDisabled",
            "my_start_of_day": "myStartOfDay",
            "project_picker_task_filter": "projectPickerTaskFilter",
            "lang": "lang",
            "multi_factor_enabled": "multiFactorEnabled",
            "theme": "theme",
            "scheduling": "scheduling",
        }


class User(BaseModel):
    id_: Optional[str]
    email: Optional[str]
    name: str
    memberships: Optional[List[Membership]]
    profile_picture: Optional[str]
    active_workspace: Optional[str]
    default_workspace: Optional[str]
    settings: Optional[Settings]
    status: Optional[str]

    class Config:
        fields = {
            "id_": "id",
            "email": "email",
            "name": "name",
            "memberships": "memberships",
            "profile_picture": "profilePicture",
            "active_workspace": "activeWorkspace",
            "default_workspace": "defaultWorkspace",
            "settings": "settings",
            "status": "status",
        }



from datetime import datetime
from pydantic import BaseModel


class BaseModel(BaseModel):
    class Config:
        allow_population_by_field_name = True

    def json_dict(self, **kwargs):
        """
        Parse dict so that it can be json serialized by Python's json module
        """
        d = super().dict(exclude_unset=True, by_alias=True, **kwargs)
        for k, v in d.items():
            if type(v) == datetime:
                d[k] = v.isoformat()[:19] + "Z"
        return d
